/// <reference types="react" />
import { AnnotationDomainType, AnnotationSpec, AnnotationType, AxisSpec, LineAnnotationDatum, LineAnnotationSpec, Position, RectAnnotationSpec, Rotation } from '../utils/specs';
import { Dimensions } from '../../../utils/dimensions';
import { AnnotationId, AxisId, GroupId } from '../../../utils/ids';
import { Scale } from '../../../utils/scales/scales';
import { Point } from '../store/chart_state';
export declare type AnnotationTooltipFormatter = (details?: string) => JSX.Element | null;
export declare type AnnotationTooltipState = AnnotationTooltipVisibleState | AnnotationTooltipHiddenState;
export interface AnnotationTooltipVisibleState {
    isVisible: true;
    annotationType: AnnotationType;
    header?: string;
    details?: string;
    anchor: {
        position?: Position;
        top: number;
        left: number;
    };
    renderTooltip?: AnnotationTooltipFormatter;
}
export interface AnnotationTooltipHiddenState {
    isVisible: false;
}
export interface AnnotationDetails {
    headerText?: string;
    detailsText?: string;
}
export interface AnnotationMarker {
    icon: JSX.Element;
    position: {
        top: number;
        left: number;
    };
    dimension: {
        width: number;
        height: number;
    };
    color: string;
}
/**
 * The path points of a line annotation.
 */
export declare type AnnotationLinePathPoints = {
    /** x1,y1 the start point anchored to the linked axis */
    start: {
        x1: number;
        y1: number;
    };
    /** x2,y2 the end point */
    end: {
        x2: number;
        y2: number;
    };
};
export interface AnnotationLineProps {
    /** the position of the start point relative to the Chart */
    anchor: {
        position: Position;
        top: number;
        left: number;
    };
    /**
     * The path points of a line annotation
     */
    linePathPoints: AnnotationLinePathPoints;
    details: AnnotationDetails;
    marker?: AnnotationMarker;
}
export interface AnnotationRectProps {
    rect: {
        x: number;
        y: number;
        width: number;
        height: number;
    };
    details?: string;
}
export declare type AnnotationDimensions = AnnotationLineProps[] | AnnotationRectProps[];
export declare function computeYDomainLineAnnotationDimensions(dataValues: LineAnnotationDatum[], yScale: Scale, chartRotation: Rotation, axisPosition: Position, chartDimensions: Dimensions, lineColor: string, marker?: JSX.Element, markerDimension?: {
    width: number;
    height: number;
}): AnnotationLineProps[];
export declare function computeXDomainLineAnnotationDimensions(dataValues: LineAnnotationDatum[], xScale: Scale, chartRotation: Rotation, axisPosition: Position, chartDimensions: Dimensions, lineColor: string, xScaleOffset: number, enableHistogramMode: boolean, marker?: JSX.Element, markerDimension?: {
    width: number;
    height: number;
}): AnnotationLineProps[];
export declare function computeLineAnnotationDimensions(annotationSpec: LineAnnotationSpec, chartDimensions: Dimensions, chartRotation: Rotation, yScales: Map<GroupId, Scale>, xScale: Scale, axisPosition: Position, xScaleOffset: number, enableHistogramMode: boolean): AnnotationLineProps[] | null;
export declare function scaleAndValidateDatum(dataValue: any, scale: Scale, alignWithTick: boolean): number | null;
export declare function computeRectAnnotationDimensions(annotationSpec: RectAnnotationSpec, yScales: Map<GroupId, Scale>, xScale: Scale, enableHistogramMode: boolean, barsPadding: number): AnnotationRectProps[] | null;
export declare function getAnnotationAxis(axesSpecs: Map<AxisId, AxisSpec>, groupId: GroupId, domainType: AnnotationDomainType, chartRotation: Rotation): Position | null;
export declare function computeClusterOffset(totalBarsInCluster: number, barsShift: number, bandwidth: number): number;
export declare function computeAnnotationDimensions(annotations: Map<AnnotationId, AnnotationSpec>, chartDimensions: Dimensions, chartRotation: Rotation, yScales: Map<GroupId, Scale>, xScale: Scale, axesSpecs: Map<AxisId, AxisSpec>, totalBarsInCluster: number, enableHistogramMode: boolean): Map<AnnotationId, AnnotationDimensions>;
/**
 * Checks if the cursorPosition is within the line annotation marker
 * @param cursorPosition the cursor position relative to the projected area
 * @param marker the line annotation marker
 */
export declare function isWithinLineMarkerBounds(cursorPosition: Point, marker: AnnotationMarker): boolean;
export declare function isVerticalAnnotationLine(isXDomainAnnotation: boolean, isHorizontalChartRotation: boolean): boolean;
export declare function getAnnotationLineTooltipXOffset(chartRotation: Rotation, axisPosition: Position): number;
export declare function getAnnotationLineTooltipYOffset(chartRotation: Rotation, axisPosition: Position): number;
export declare function isXDomain(domainType: AnnotationDomainType): boolean;
export declare function computeLineAnnotationTooltipState(cursorPosition: Point, annotationLines: AnnotationLineProps[], groupId: GroupId, domainType: AnnotationDomainType, axesSpecs: Map<AxisId, AxisSpec>): AnnotationTooltipState;
export declare function isWithinRectBounds(cursorPosition: Point, { startX, endX, startY, endY }: {
    startX: number;
    endX: number;
    startY: number;
    endY: number;
}): boolean;
export declare function isRightRectTooltip(chartRotation: Rotation, cursorPosition: Point, chartWidth: number): boolean;
export declare function isBottomRectTooltip(chartRotation: Rotation, cursorPosition: Point, chartHeight: number): boolean;
export declare function computeRectTooltipLeft(chartRotation: Rotation, isRightTooltip: boolean, { startX, endX }: {
    startX: number;
    endX: number;
}, cursorX: number, chartWidth: number): number;
export declare function computeRectTooltipTop(chartRotation: Rotation, isBottomTooltip: boolean, { startX, endX }: {
    startX: number;
    endX: number;
}, cursorY: number, chartHeight: number): number;
export declare function getRotatedCursor(
/** the cursor position relative to the projection area */
cursorPosition: Point, chartDimensions: Dimensions, chartRotation: Rotation): Point;
export declare function computeRectAnnotationTooltipState(
/** the cursor position relative to the projection area */
cursorPosition: Point, annotationRects: AnnotationRectProps[], chartRotation: Rotation, chartDimensions: Dimensions, renderTooltip?: AnnotationTooltipFormatter): AnnotationTooltipState;
export declare function computeAnnotationTooltipState(cursorPosition: Point, annotationDimensions: Map<AnnotationId, any>, annotationSpecs: Map<AnnotationId, AnnotationSpec>, chartRotation: Rotation, axesSpecs: Map<AxisId, AxisSpec>, chartDimensions: Dimensions): AnnotationTooltipState | null;
