"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var specs_1 = require("../utils/specs");
var axis_utils_1 = require("../utils/axis_utils");
var scales_1 = require("../../../utils/scales/scales");
var utils_1 = require("../store/utils");
function computeYDomainLineAnnotationDimensions(dataValues, yScale, chartRotation, axisPosition, chartDimensions, lineColor, marker, markerDimension) {
    if (markerDimension === void 0) { markerDimension = { width: 0, height: 0 }; }
    var chartHeight = chartDimensions.height;
    var chartWidth = chartDimensions.width;
    var isHorizontalChartRotation = utils_1.isHorizontalRotation(chartRotation);
    var lineProps = [];
    dataValues.forEach(function (datum) {
        var dataValue = datum.dataValue;
        // avoid rendering invalid annotation value
        if (dataValue === null || dataValue === undefined || dataValue === '') {
            return;
        }
        var annotationValueYposition = yScale.scale(dataValue);
        // avoid rendering non scalable annotation values
        if (isNaN(annotationValueYposition)) {
            return;
        }
        var _a = __read(yScale.domain, 2), domainStart = _a[0], domainEnd = _a[1];
        // avoid rendering annotation with values outside the scale domain
        if (domainStart > dataValue || domainEnd < dataValue) {
            return;
        }
        var anchor = {
            position: axisPosition,
            top: 0,
            left: 0,
        };
        var markerPosition = { top: 0, left: 0 };
        var linePathPoints = {
            start: { x1: 0, y1: 0 },
            end: { x2: 0, y2: 0 },
        };
        // the Y axis is vertical, X axis is horizontal  y|--x--|y
        if (isHorizontalChartRotation) {
            // y|__x__
            if (axisPosition === specs_1.Position.Left) {
                anchor.left = 0;
                markerPosition.left = -markerDimension.width;
                linePathPoints.start.x1 = 0;
                linePathPoints.end.x2 = chartWidth;
                // __x__|y
            }
            else {
                anchor.left = chartWidth;
                markerPosition.left = chartWidth;
                linePathPoints.start.x1 = chartWidth;
                linePathPoints.end.x2 = 0;
            }
            // __x__
            if (chartRotation === 0) {
                anchor.top = annotationValueYposition;
                markerPosition.top = annotationValueYposition - markerDimension.height / 2;
                // ¯¯x¯¯
            }
            else {
                anchor.top = chartHeight - annotationValueYposition;
                markerPosition.top = chartHeight - annotationValueYposition - markerDimension.height / 2;
            }
            linePathPoints.start.y1 = annotationValueYposition;
            linePathPoints.end.y2 = annotationValueYposition;
            // the Y axis is horizontal, X axis is vertical x|--y--|x
        }
        else {
            // ¯¯y¯¯
            if (axisPosition === specs_1.Position.Top) {
                anchor.top = 0;
                markerPosition.top = -markerDimension.height;
                linePathPoints.start.x1 = 0;
                linePathPoints.end.x2 = chartHeight;
                // __y__
            }
            else {
                anchor.top = chartHeight;
                markerPosition.top = chartHeight;
                linePathPoints.start.x1 = chartHeight;
                linePathPoints.end.x2 = 0;
            }
            // __y__|x
            if (chartRotation === -90) {
                anchor.left = annotationValueYposition;
                markerPosition.left = annotationValueYposition - markerDimension.width / 2;
                // x|__y__
            }
            else {
                anchor.left = chartWidth - annotationValueYposition;
                markerPosition.left = chartWidth - annotationValueYposition - markerDimension.width / 2;
            }
            linePathPoints.start.y1 = annotationValueYposition;
            linePathPoints.end.y2 = annotationValueYposition;
        }
        var annotationMarker = marker
            ? {
                icon: marker,
                color: lineColor,
                dimension: markerDimension,
                position: markerPosition,
            }
            : undefined;
        var lineProp = {
            anchor: anchor,
            linePathPoints: linePathPoints,
            marker: annotationMarker,
            details: {
                detailsText: datum.details,
                headerText: datum.header || dataValue.toString(),
            },
        };
        lineProps.push(lineProp);
    });
    return lineProps;
}
exports.computeYDomainLineAnnotationDimensions = computeYDomainLineAnnotationDimensions;
function computeXDomainLineAnnotationDimensions(dataValues, xScale, chartRotation, axisPosition, chartDimensions, lineColor, xScaleOffset, enableHistogramMode, marker, markerDimension) {
    if (markerDimension === void 0) { markerDimension = { width: 0, height: 0 }; }
    var chartHeight = chartDimensions.height;
    var chartWidth = chartDimensions.width;
    var lineProps = [];
    var isHorizontalChartRotation = utils_1.isHorizontalRotation(chartRotation);
    var alignWithTick = xScale.bandwidth > 0 && !enableHistogramMode;
    dataValues.forEach(function (datum) {
        var dataValue = datum.dataValue;
        var scaledXValue = scaleAndValidateDatum(dataValue, xScale, alignWithTick);
        if (scaledXValue == null) {
            return;
        }
        var offset = xScale.bandwidth / 2 - xScaleOffset;
        var annotationValueXposition = scaledXValue + offset;
        var markerPosition = { top: 0, left: 0 };
        var linePathPoints = {
            start: { x1: 0, y1: 0 },
            end: { x2: 0, y2: 0 },
        };
        var anchor = {
            position: axisPosition,
            top: 0,
            left: 0,
        };
        // the Y axis is vertical, X axis is horizontal  y|--x--|y
        if (isHorizontalChartRotation) {
            // __x__
            if (axisPosition === specs_1.Position.Bottom) {
                linePathPoints.start.y1 = chartHeight;
                linePathPoints.end.y2 = 0;
                anchor.top = chartHeight;
                markerPosition.top = chartHeight;
                // ¯¯x¯¯
            }
            else {
                linePathPoints.start.y1 = 0;
                linePathPoints.end.y2 = chartHeight;
                anchor.top = 0;
                markerPosition.top = 0 - markerDimension.height;
            }
            // __x__
            if (chartRotation === 0) {
                anchor.left = annotationValueXposition;
                markerPosition.left = annotationValueXposition - markerDimension.width / 2;
                // ¯¯x¯¯
            }
            else {
                anchor.left = chartWidth - annotationValueXposition;
                markerPosition.left = chartWidth - annotationValueXposition - markerDimension.width / 2;
            }
            linePathPoints.start.x1 = annotationValueXposition;
            linePathPoints.end.x2 = annotationValueXposition;
            // the Y axis is horizontal, X axis is vertical x|--y--|x
        }
        else {
            // x|--y--
            if (axisPosition === specs_1.Position.Left) {
                anchor.left = 0;
                markerPosition.left = -markerDimension.width;
                linePathPoints.start.x1 = annotationValueXposition;
                linePathPoints.end.x2 = annotationValueXposition;
                // --y--|x
            }
            else {
                anchor.left = chartWidth;
                markerPosition.left = chartWidth;
                linePathPoints.start.x1 = annotationValueXposition;
                linePathPoints.end.x2 = annotationValueXposition;
            }
            // __y__|x
            if (chartRotation === -90) {
                anchor.top = chartHeight - annotationValueXposition;
                markerPosition.top = chartHeight - annotationValueXposition - markerDimension.height / 2;
                linePathPoints.start.y1 = 0;
                linePathPoints.end.y2 = chartWidth;
                // x|__y__
            }
            else {
                anchor.top = annotationValueXposition;
                markerPosition.top = annotationValueXposition - markerDimension.height / 2;
                linePathPoints.start.y1 = 0;
                linePathPoints.end.y2 = chartWidth;
            }
        }
        var annotationMarker = marker
            ? {
                icon: marker,
                color: lineColor,
                dimension: markerDimension,
                position: markerPosition,
            }
            : undefined;
        var lineProp = {
            anchor: anchor,
            linePathPoints: linePathPoints,
            details: {
                detailsText: datum.details,
                headerText: datum.header || dataValue.toString(),
            },
            marker: annotationMarker,
        };
        lineProps.push(lineProp);
    });
    return lineProps;
}
exports.computeXDomainLineAnnotationDimensions = computeXDomainLineAnnotationDimensions;
function computeLineAnnotationDimensions(annotationSpec, chartDimensions, chartRotation, yScales, xScale, axisPosition, xScaleOffset, enableHistogramMode) {
    var domainType = annotationSpec.domainType, dataValues = annotationSpec.dataValues, marker = annotationSpec.marker, markerDimensions = annotationSpec.markerDimensions, hideLines = annotationSpec.hideLines;
    if (hideLines) {
        return null;
    }
    // this type is guaranteed as this has been merged with default
    var lineStyle = annotationSpec.style;
    var lineColor = lineStyle.line.stroke;
    if (domainType === specs_1.AnnotationDomainTypes.XDomain) {
        return computeXDomainLineAnnotationDimensions(dataValues, xScale, chartRotation, axisPosition, chartDimensions, lineColor, xScaleOffset, enableHistogramMode, marker, markerDimensions);
    }
    var groupId = annotationSpec.groupId;
    var yScale = yScales.get(groupId);
    if (!yScale) {
        return null;
    }
    return computeYDomainLineAnnotationDimensions(dataValues, yScale, chartRotation, axisPosition, chartDimensions, lineColor, marker, markerDimensions);
}
exports.computeLineAnnotationDimensions = computeLineAnnotationDimensions;
function scaleAndValidateDatum(dataValue, scale, alignWithTick) {
    var isContinuous = scale.type !== scales_1.ScaleType.Ordinal;
    var scaledValue = scale.scale(dataValue);
    // d3.scale will return 0 for '', rendering the line incorrectly at 0
    if (isNaN(scaledValue) || (isContinuous && dataValue === '')) {
        return null;
    }
    if (isContinuous) {
        var _a = __read(scale.domain, 2), domainStart = _a[0], domainEnd = _a[1];
        // if we're not aligning the ticks, we need to extend the domain by one more tick for histograms
        var domainEndOffset = alignWithTick ? 0 : scale.minInterval;
        if (domainStart > dataValue || domainEnd + domainEndOffset < dataValue) {
            return null;
        }
    }
    return scaledValue;
}
exports.scaleAndValidateDatum = scaleAndValidateDatum;
function computeRectAnnotationDimensions(annotationSpec, yScales, xScale, enableHistogramMode, barsPadding) {
    var dataValues = annotationSpec.dataValues;
    var groupId = annotationSpec.groupId;
    var yScale = yScales.get(groupId);
    if (!yScale) {
        return null;
    }
    var xDomain = xScale.domain;
    var yDomain = yScale.domain;
    var lastX = xDomain[xDomain.length - 1];
    var xMinInterval = xScale.minInterval;
    var rectsProps = [];
    dataValues.forEach(function (dataValue) {
        var _a = dataValue.coordinates, x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
        // if everything is null, return; otherwise we coerce the other coordinates
        if (x0 == null && x1 == null && y0 == null && y1 == null) {
            return;
        }
        if (x1 == null) {
            // if x1 is defined, we want the rect to draw to the end of the scale
            // if we're in histogram mode, extend domain end by min interval
            x1 = enableHistogramMode && !xScale.isSingleValue() ? lastX + xMinInterval : lastX;
        }
        if (x0 == null) {
            // if x0 is defined, we want the rect to draw to the start of the scale
            x0 = xDomain[0];
        }
        if (y0 == null) {
            // if y0 is defined, we want the rect to draw to the end of the scale
            y0 = yDomain[yDomain.length - 1];
        }
        if (y1 == null) {
            // if y1 is defined, we want the rect to draw to the start of the scale
            y1 = yDomain[0];
        }
        var alignWithTick = xScale.bandwidth > 0 && !enableHistogramMode;
        var x0Scaled = scaleAndValidateDatum(x0, xScale, alignWithTick);
        var x1Scaled = scaleAndValidateDatum(x1, xScale, alignWithTick);
        var y0Scaled = scaleAndValidateDatum(y0, yScale, false);
        var y1Scaled = scaleAndValidateDatum(y1, yScale, false);
        // TODO: surface this as a warning
        if (x0Scaled === null || x1Scaled === null || y0Scaled === null || y1Scaled === null) {
            return;
        }
        var xOffset = 0;
        if (xScale.bandwidth > 0) {
            var xBand = xScale.bandwidth / (1 - xScale.barsPadding);
            xOffset = enableHistogramMode ? (xBand - xScale.bandwidth) / 2 : barsPadding;
        }
        x0Scaled = x0Scaled - xOffset;
        x1Scaled = x1Scaled - xOffset;
        var minX = Math.min(x0Scaled, x1Scaled);
        var minY = Math.min(y0Scaled, y1Scaled);
        var deltaX = Math.abs(x0Scaled - x1Scaled);
        var deltaY = Math.abs(y0Scaled - y1Scaled);
        var xOrigin = minX;
        var yOrigin = minY;
        var width = deltaX;
        var height = deltaY;
        var rectDimensions = {
            x: xOrigin,
            y: yOrigin,
            width: width,
            height: height,
        };
        rectsProps.push({
            rect: rectDimensions,
            details: dataValue.details,
        });
    });
    return rectsProps;
}
exports.computeRectAnnotationDimensions = computeRectAnnotationDimensions;
function getAnnotationAxis(axesSpecs, groupId, domainType, chartRotation) {
    var _a = utils_1.getAxesSpecForSpecId(axesSpecs, groupId), xAxis = _a.xAxis, yAxis = _a.yAxis;
    var isHorizontalRotated = utils_1.isHorizontalRotation(chartRotation);
    var isXDomainAnnotation = isXDomain(domainType);
    var annotationAxis = isXDomainAnnotation ? xAxis : yAxis;
    var rotatedAnnotation = isHorizontalRotated ? annotationAxis : isXDomainAnnotation ? yAxis : xAxis;
    return rotatedAnnotation ? rotatedAnnotation.position : null;
}
exports.getAnnotationAxis = getAnnotationAxis;
function computeClusterOffset(totalBarsInCluster, barsShift, bandwidth) {
    if (totalBarsInCluster > 1) {
        return barsShift - bandwidth / 2;
    }
    return 0;
}
exports.computeClusterOffset = computeClusterOffset;
function computeAnnotationDimensions(annotations, chartDimensions, chartRotation, yScales, xScale, axesSpecs, totalBarsInCluster, enableHistogramMode) {
    var annotationDimensions = new Map();
    var barsShift = (totalBarsInCluster * xScale.bandwidth) / 2;
    var band = xScale.bandwidth / (1 - xScale.barsPadding);
    var halfPadding = (band - xScale.bandwidth) / 2;
    var barsPadding = halfPadding * totalBarsInCluster;
    var clusterOffset = computeClusterOffset(totalBarsInCluster, barsShift, xScale.bandwidth);
    // Annotations should always align with the axis line in histogram mode
    var xScaleOffset = utils_1.computeXScaleOffset(xScale, enableHistogramMode, specs_1.HistogramModeAlignments.Start);
    annotations.forEach(function (annotationSpec, annotationId) {
        if (specs_1.isLineAnnotation(annotationSpec)) {
            var groupId = annotationSpec.groupId, domainType = annotationSpec.domainType;
            var annotationAxisPosition = getAnnotationAxis(axesSpecs, groupId, domainType, chartRotation);
            if (!annotationAxisPosition) {
                return;
            }
            var dimensions = computeLineAnnotationDimensions(annotationSpec, chartDimensions, chartRotation, yScales, xScale, annotationAxisPosition, xScaleOffset - clusterOffset, enableHistogramMode);
            if (dimensions) {
                annotationDimensions.set(annotationId, dimensions);
            }
        }
        else if (specs_1.isRectAnnotation(annotationSpec)) {
            var dimensions = computeRectAnnotationDimensions(annotationSpec, yScales, xScale, enableHistogramMode, barsPadding);
            if (dimensions) {
                annotationDimensions.set(annotationId, dimensions);
            }
        }
    });
    return annotationDimensions;
}
exports.computeAnnotationDimensions = computeAnnotationDimensions;
/**
 * Checks if the cursorPosition is within the line annotation marker
 * @param cursorPosition the cursor position relative to the projected area
 * @param marker the line annotation marker
 */
function isWithinLineMarkerBounds(cursorPosition, marker) {
    var _a = marker.position, top = _a.top, left = _a.left;
    var _b = marker.dimension, width = _b.width, height = _b.height;
    var markerRect = { startX: left, startY: top, endX: left + width, endY: top + height };
    return isWithinRectBounds(cursorPosition, markerRect);
}
exports.isWithinLineMarkerBounds = isWithinLineMarkerBounds;
function isVerticalAnnotationLine(isXDomainAnnotation, isHorizontalChartRotation) {
    if (isXDomainAnnotation) {
        return isHorizontalChartRotation;
    }
    return !isHorizontalChartRotation;
}
exports.isVerticalAnnotationLine = isVerticalAnnotationLine;
function getAnnotationLineTooltipXOffset(chartRotation, axisPosition) {
    var xOffset = 0;
    var isChartHorizontalRotation = utils_1.isHorizontalRotation(chartRotation);
    if (axis_utils_1.isHorizontalAxis(axisPosition)) {
        xOffset = isChartHorizontalRotation ? 50 : 0;
    }
    else {
        xOffset = isChartHorizontalRotation ? (axisPosition === specs_1.Position.Right ? 100 : 0) : 50;
    }
    return xOffset;
}
exports.getAnnotationLineTooltipXOffset = getAnnotationLineTooltipXOffset;
function getAnnotationLineTooltipYOffset(chartRotation, axisPosition) {
    var yOffset = 0;
    var isChartHorizontalRotation = utils_1.isHorizontalRotation(chartRotation);
    if (axis_utils_1.isHorizontalAxis(axisPosition)) {
        yOffset = isChartHorizontalRotation ? (axisPosition === specs_1.Position.Top ? 0 : 100) : 50;
    }
    else {
        yOffset = isChartHorizontalRotation ? 50 : 100;
    }
    return yOffset;
}
exports.getAnnotationLineTooltipYOffset = getAnnotationLineTooltipYOffset;
function isXDomain(domainType) {
    return domainType === specs_1.AnnotationDomainTypes.XDomain;
}
exports.isXDomain = isXDomain;
function computeLineAnnotationTooltipState(cursorPosition, annotationLines, groupId, domainType, axesSpecs) {
    var _a = utils_1.getAxesSpecForSpecId(axesSpecs, groupId), xAxis = _a.xAxis, yAxis = _a.yAxis;
    var isXDomainAnnotation = isXDomain(domainType);
    var annotationAxis = isXDomainAnnotation ? xAxis : yAxis;
    if (!annotationAxis) {
        return {
            isVisible: false,
        };
    }
    var totalAnnotationLines = annotationLines.length;
    for (var i = 0; i < totalAnnotationLines; i++) {
        var line = annotationLines[i];
        var isWithinBounds = line.marker && isWithinLineMarkerBounds(cursorPosition, line.marker);
        if (isWithinBounds) {
            return __assign(__assign({ annotationType: specs_1.AnnotationTypes.Line, isVisible: true, anchor: __assign({}, line.anchor) }, (line.details && { header: line.details.headerText })), (line.details && { details: line.details.detailsText }));
        }
    }
    return {
        isVisible: false,
    };
}
exports.computeLineAnnotationTooltipState = computeLineAnnotationTooltipState;
function isWithinRectBounds(cursorPosition, _a) {
    var startX = _a.startX, endX = _a.endX, startY = _a.startY, endY = _a.endY;
    var withinXBounds = cursorPosition.x >= startX && cursorPosition.x <= endX;
    var withinYBounds = cursorPosition.y >= startY && cursorPosition.y <= endY;
    return withinXBounds && withinYBounds;
}
exports.isWithinRectBounds = isWithinRectBounds;
function isRightRectTooltip(chartRotation, cursorPosition, chartWidth) {
    var xPosition = utils_1.isHorizontalRotation(chartRotation) ? cursorPosition.x : cursorPosition.y;
    return chartRotation === -90 ? xPosition > chartWidth / 2 : xPosition < chartWidth / 2;
}
exports.isRightRectTooltip = isRightRectTooltip;
function isBottomRectTooltip(chartRotation, cursorPosition, chartHeight) {
    var yPosition = utils_1.isHorizontalRotation(chartRotation) ? cursorPosition.y : cursorPosition.x;
    return chartRotation === 180 ? yPosition > chartHeight / 2 : yPosition < chartHeight / 2;
}
exports.isBottomRectTooltip = isBottomRectTooltip;
function computeRectTooltipLeft(chartRotation, isRightTooltip, _a, cursorX, chartWidth) {
    var startX = _a.startX, endX = _a.endX;
    var isHorizontalChartRotation = utils_1.isHorizontalRotation(chartRotation);
    var horizontalLeft = isRightTooltip ? endX : startX;
    return isHorizontalChartRotation ? (chartRotation === 180 ? chartWidth - horizontalLeft : horizontalLeft) : cursorX;
}
exports.computeRectTooltipLeft = computeRectTooltipLeft;
function computeRectTooltipTop(chartRotation, isBottomTooltip, _a, cursorY, chartHeight) {
    var startX = _a.startX, endX = _a.endX;
    var isHorizontalChartRotation = utils_1.isHorizontalRotation(chartRotation);
    var verticalTop = isBottomTooltip ? endX : startX;
    return isHorizontalChartRotation ? cursorY : chartRotation === -90 ? chartHeight - verticalTop : verticalTop;
}
exports.computeRectTooltipTop = computeRectTooltipTop;
function getRotatedCursor(
/** the cursor position relative to the projection area */
cursorPosition, chartDimensions, chartRotation) {
    var x = cursorPosition.x, y = cursorPosition.y;
    var height = chartDimensions.height, width = chartDimensions.width;
    switch (chartRotation) {
        case 0:
            return { x: x, y: y };
        case 90:
            return { x: y, y: width - x };
        case -90:
            return { x: height - y, y: x };
        case 180:
            return { x: width - x, y: height - y };
    }
}
exports.getRotatedCursor = getRotatedCursor;
function computeRectAnnotationTooltipState(
/** the cursor position relative to the projection area */
cursorPosition, annotationRects, chartRotation, chartDimensions, renderTooltip) {
    var rotatedCursorPosition = getRotatedCursor(cursorPosition, chartDimensions, chartRotation);
    var totalAnnotationRect = annotationRects.length;
    for (var i = 0; i < totalAnnotationRect; i++) {
        var rectProps = annotationRects[i];
        var rect = rectProps.rect, details = rectProps.details;
        var startX = rect.x;
        var endX = startX + rect.width;
        var startY = rect.y;
        var endY = startY + rect.height;
        var isWithinBounds = isWithinRectBounds(rotatedCursorPosition, { startX: startX, endX: endX, startY: startY, endY: endY });
        if (isWithinBounds) {
            return __assign(__assign({ isVisible: true, annotationType: specs_1.AnnotationTypes.Rectangle, anchor: {
                    left: rotatedCursorPosition.x,
                    top: rotatedCursorPosition.y,
                } }, (details && { details: details })), (renderTooltip && { renderTooltip: renderTooltip }));
        }
    }
    return {
        isVisible: false,
    };
}
exports.computeRectAnnotationTooltipState = computeRectAnnotationTooltipState;
function computeAnnotationTooltipState(cursorPosition, annotationDimensions, annotationSpecs, chartRotation, axesSpecs, chartDimensions) {
    var e_1, _a;
    try {
        for (var annotationDimensions_1 = __values(annotationDimensions), annotationDimensions_1_1 = annotationDimensions_1.next(); !annotationDimensions_1_1.done; annotationDimensions_1_1 = annotationDimensions_1.next()) {
            var _b = __read(annotationDimensions_1_1.value, 2), annotationId = _b[0], annotationDimension = _b[1];
            var spec = annotationSpecs.get(annotationId);
            if (!spec || spec.hideTooltips) {
                continue;
            }
            var groupId = spec.groupId;
            if (specs_1.isLineAnnotation(spec)) {
                if (spec.hideLines) {
                    continue;
                }
                var lineAnnotationTooltipState = computeLineAnnotationTooltipState(cursorPosition, annotationDimension, groupId, spec.domainType, axesSpecs);
                if (lineAnnotationTooltipState.isVisible) {
                    return lineAnnotationTooltipState;
                }
            }
            else if (specs_1.isRectAnnotation(spec)) {
                var rectAnnotationTooltipState = computeRectAnnotationTooltipState(cursorPosition, annotationDimension, chartRotation, chartDimensions, spec.renderTooltip);
                if (rectAnnotationTooltipState.isVisible) {
                    return rectAnnotationTooltipState;
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (annotationDimensions_1_1 && !annotationDimensions_1_1.done && (_a = annotationDimensions_1.return)) _a.call(annotationDimensions_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return null;
}
exports.computeAnnotationTooltipState = computeAnnotationTooltipState;
//# sourceMappingURL=annotation_utils.js.map