"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../store/utils");
var commons_1 = require("../../../utils/commons");
var series_1 = require("../utils/series");
var specs_1 = require("../utils/specs");
var tooltip_1 = require("../tooltip/tooltip");
function getPostfix(spec) {
    if (specs_1.isAreaSeriesSpec(spec) || specs_1.isBarSeriesSpec(spec)) {
        var _a = spec.y0AccessorFormat, y0AccessorFormat = _a === void 0 ? tooltip_1.Y0_ACCESSOR_POSTFIX : _a, _b = spec.y1AccessorFormat, y1AccessorFormat = _b === void 0 ? tooltip_1.Y1_ACCESSOR_POSTFIX : _b;
        return {
            y0AccessorFormat: y0AccessorFormat,
            y1AccessorFormat: y1AccessorFormat,
        };
    }
    return {};
}
exports.getPostfix = getPostfix;
function computeLegend(seriesColor, seriesColorMap, specs, defaultColor, axesSpecs, deselectedDataSeries) {
    var legendItems = new Map();
    var sortedSeriesColors = series_1.getSortedDataSeriesColorsValuesMap(seriesColor);
    sortedSeriesColors.forEach(function (series, key) {
        var banded = series.banded, specId = series.specId, lastValue = series.lastValue, colorValues = series.colorValues;
        var spec = specs.get(specId);
        var color = seriesColorMap.get(key) || defaultColor;
        var hasSingleSeries = seriesColor.size === 1;
        var label = getSeriesColorLabel(colorValues, hasSingleSeries, spec);
        var isSeriesVisible = deselectedDataSeries ? series_1.findDataSeriesByColorValues(deselectedDataSeries, series) < 0 : true;
        if (!label || !spec) {
            return;
        }
        // Use this to get axis spec w/ tick formatter
        var yAxis = utils_1.getAxesSpecForSpecId(axesSpecs, spec.groupId).yAxis;
        var formatter = yAxis ? yAxis.tickFormat : commons_1.identity;
        var hideInLegend = spec.hideInLegend;
        var legendItem = __assign({ key: key,
            color: color,
            label: label,
            banded: banded, value: series, isSeriesVisible: isSeriesVisible, isLegendItemVisible: !hideInLegend, displayValue: {
                raw: {
                    y0: lastValue && lastValue.y0 !== null ? lastValue.y0 : null,
                    y1: lastValue && lastValue.y1 !== null ? lastValue.y1 : null,
                },
                formatted: {
                    y0: isSeriesVisible && lastValue && lastValue.y0 !== null ? formatter(lastValue.y0) : null,
                    y1: isSeriesVisible && lastValue && lastValue.y1 !== null ? formatter(lastValue.y1) : null,
                },
            } }, getPostfix(spec));
        legendItems.set(key, legendItem);
    });
    return legendItems;
}
exports.computeLegend = computeLegend;
function getSeriesColorLabel(colorValues, hasSingleSeries, spec) {
    var label = '';
    if (hasSingleSeries || colorValues.length === 0 || colorValues[0] == null) {
        if (!spec) {
            return;
        }
        if (spec.splitSeriesAccessors && colorValues.length > 0 && colorValues[0] !== null) {
            label = colorValues.join(' - ');
        }
        else {
            label = spec.name || "" + spec.id;
        }
    }
    else {
        label = colorValues.join(' - ');
    }
    return label;
}
exports.getSeriesColorLabel = getSeriesColorLabel;
//# sourceMappingURL=legend.js.map