import { AreaSeriesStyle, AreaStyle, LineSeriesStyle, LineStyle, PointStyle, SharedGeometryStyle, BarSeriesStyle } from '../../../utils/themes/theme';
import { SpecId } from '../../../utils/ids';
import { Scale } from '../../../utils/scales/scales';
import { CurveType } from '../../../utils/curves';
import { LegendItem } from '../legend/legend';
import { DataSeriesDatum } from '../utils/series';
import { DisplayValueSpec, BarStyleAccessor, PointStyleAccessor } from '../utils/specs';
export interface GeometryId {
    specId: SpecId;
    seriesKey: any[];
}
/**
 * The accessor type
 */
export declare const AccessorType: Readonly<{
    Y0: "y0";
    Y1: "y1";
}>;
export declare type AccessorType = typeof AccessorType.Y0 | typeof AccessorType.Y1;
export interface GeometryValue {
    y: any;
    x: any;
    accessor: AccessorType;
}
/** Shared style properties for varies geometries */
export interface GeometryStyle {
    /**
     * Opacity multiplier
     *
     * if set to `0.5` all given opacities will be halfed
     */
    opacity: number;
}
export declare type IndexedGeometry = PointGeometry | BarGeometry;
export interface PointGeometry {
    x: number;
    y: number;
    radius: number;
    color: string;
    transform: {
        x: number;
        y: number;
    };
    geometryId: GeometryId;
    value: GeometryValue;
    styleOverrides?: Partial<PointStyle>;
}
export interface BarGeometry {
    x: number;
    y: number;
    width: number;
    height: number;
    color: string;
    displayValue?: {
        text: any;
        width: number;
        height: number;
        hideClippedValue?: boolean;
        isValueContainedInElement?: boolean;
    };
    geometryId: GeometryId;
    value: GeometryValue;
    seriesStyle: BarSeriesStyle;
}
export interface LineGeometry {
    line: string;
    points: PointGeometry[];
    color: string;
    transform: {
        x: number;
        y: number;
    };
    geometryId: GeometryId;
    seriesLineStyle: LineStyle;
    seriesPointStyle: PointStyle;
}
export interface AreaGeometry {
    area: string;
    lines: string[];
    points: PointGeometry[];
    color: string;
    transform: {
        x: number;
        y: number;
    };
    geometryId: GeometryId;
    seriesAreaStyle: AreaStyle;
    seriesAreaLineStyle: LineStyle;
    seriesPointStyle: PointStyle;
    isStacked: boolean;
}
export declare function isPointGeometry(ig: IndexedGeometry): ig is PointGeometry;
export declare function isBarGeometry(ig: IndexedGeometry): ig is BarGeometry;
export declare function mutableIndexedGeometryMapUpsert(mutableGeometriesIndex: Map<any, IndexedGeometry[]>, key: any, geometry: IndexedGeometry | IndexedGeometry[]): void;
export declare function getPointStyleOverrides(datum: DataSeriesDatum, geometryId: GeometryId, pointStyleAccessor?: PointStyleAccessor): Partial<PointStyle> | undefined;
export declare function getBarStyleOverrides(datum: DataSeriesDatum, geometryId: GeometryId, seriesStyle: BarSeriesStyle, styleAccessor?: BarStyleAccessor): BarSeriesStyle;
export declare function renderPoints(shift: number, dataset: DataSeriesDatum[], xScale: Scale, yScale: Scale, color: string, specId: SpecId, hasY0Accessors: boolean, seriesKey: any[], styleAccessor?: PointStyleAccessor): {
    pointGeometries: PointGeometry[];
    indexedGeometries: Map<any, IndexedGeometry[]>;
};
export declare function renderBars(orderIndex: number, dataset: DataSeriesDatum[], xScale: Scale, yScale: Scale, color: string, specId: SpecId, seriesKey: any[], sharedSeriesStyle: BarSeriesStyle, displayValueSettings?: DisplayValueSpec, styleAccessor?: BarStyleAccessor, minBarHeight?: number): {
    barGeometries: BarGeometry[];
    indexedGeometries: Map<any, IndexedGeometry[]>;
};
export declare function renderLine(shift: number, dataset: DataSeriesDatum[], xScale: Scale, yScale: Scale, color: string, curve: CurveType, specId: SpecId, hasY0Accessors: boolean, seriesKey: any[], xScaleOffset: number, seriesStyle: LineSeriesStyle, pointStyleAccessor?: PointStyleAccessor): {
    lineGeometry: LineGeometry;
    indexedGeometries: Map<any, IndexedGeometry[]>;
};
export declare function renderArea(shift: number, dataset: DataSeriesDatum[], xScale: Scale, yScale: Scale, color: string, curve: CurveType, specId: SpecId, hasY0Accessors: boolean, seriesKey: any[], xScaleOffset: number, seriesStyle: AreaSeriesStyle, isStacked?: boolean, pointStyleAccessor?: PointStyleAccessor): {
    areaGeometry: AreaGeometry;
    indexedGeometries: Map<any, IndexedGeometry[]>;
};
export declare function getGeometryStyle(geometryId: GeometryId, highlightedLegendItem: LegendItem | null, sharedGeometryStyle: SharedGeometryStyle, individualHighlight?: {
    [key: string]: boolean;
}): GeometryStyle;
export declare function isPointOnGeometry(xCoordinate: number, yCoordinate: number, indexedGeometry: BarGeometry | PointGeometry): boolean;
export declare function getGeometryIdKey(geometryId: GeometryId, prefix?: string, postfix?: string): string;
