import { XDomain } from '../domains/x_domain';
import { YDomain } from '../domains/y_domain';
import { AreaGeometry, BarGeometry, IndexedGeometry, LineGeometry, PointGeometry } from '../rendering/rendering';
import { DataSeries, DataSeriesColorsValues, FormattedDataSeries } from '../utils/series';
import { AxisSpec, BasicSeriesSpec, DomainRange, HistogramModeAlignment, Rotation } from '../utils/specs';
import { Theme } from '../../../utils/themes/theme';
import { Dimensions } from '../../../utils/dimensions';
import { Domain } from '../../../utils/domain';
import { AxisId, GroupId, SpecId } from '../../../utils/ids';
import { Scale } from '../../../utils/scales/scales';
import { SeriesDomainsAndData } from './chart_state';
import { LegendItem } from '../legend/legend';
export interface Transform {
    x: number;
    y: number;
    rotate: number;
}
export interface BrushExtent {
    minX: number;
    minY: number;
    maxX: number;
    maxY: number;
}
export interface GeometriesCounts {
    points: number;
    bars: number;
    areas: number;
    areasPoints: number;
    lines: number;
    linePoints: number;
}
export declare function updateDeselectedDataSeries(series: DataSeriesColorsValues[] | null, value: DataSeriesColorsValues): DataSeriesColorsValues[];
export declare function getUpdatedCustomSeriesColors(seriesSpecs: Map<SpecId, BasicSeriesSpec>): Map<string, string>;
export interface LastValues {
    y0: number | null;
    y1: number | null;
}
export declare function getLastValues(formattedDataSeries: {
    stacked: FormattedDataSeries[];
    nonStacked: FormattedDataSeries[];
}): Map<string, LastValues>;
/**
 * Compute data domains for all specified specs.
 * @param seriesSpecs a map of all the series specs
 * @param customYDomainsByGroupId custom Y domains grouped by GroupId
 * @param customXDomain if specified in <Settings />, the custom X domain
 * @param deselectedDataSeries is optional; if not supplied,
 * then all series will be factored into computations. Otherwise, selectedDataSeries
 * is used to restrict the computation for just the selected series
 * @returns `SeriesDomainsAndData`
 */
export declare function computeSeriesDomains(seriesSpecs: Map<SpecId, BasicSeriesSpec>, customYDomainsByGroupId: Map<GroupId, DomainRange>, customXDomain?: DomainRange | Domain, deselectedDataSeries?: DataSeriesColorsValues[] | null): SeriesDomainsAndData;
export declare function computeSeriesGeometries(seriesSpecs: Map<SpecId, BasicSeriesSpec>, xDomain: XDomain, yDomain: YDomain[], formattedDataSeries: {
    stacked: FormattedDataSeries[];
    nonStacked: FormattedDataSeries[];
}, seriesColorMap: Map<string, string>, chartTheme: Theme, chartDims: Dimensions, chartRotation: Rotation, axesSpecs: Map<AxisId, AxisSpec>, enableHistogramMode: boolean): {
    scales: {
        xScale: Scale;
        yScales: Map<GroupId, Scale>;
    };
    geometries: {
        points: PointGeometry[];
        bars: BarGeometry[];
        areas: AreaGeometry[];
        lines: LineGeometry[];
    };
    geometriesIndex: Map<any, IndexedGeometry[]>;
    geometriesCounts: GeometriesCounts;
};
export declare function setBarSeriesAccessors(isHistogramMode: boolean, seriesSpecs: Map<SpecId, BasicSeriesSpec>): void;
export declare function isHistogramModeEnabled(seriesSpecs: Map<SpecId, BasicSeriesSpec>): boolean;
export declare function computeXScaleOffset(xScale: Scale, enableHistogramMode: boolean, histogramModeAlignment?: HistogramModeAlignment): number;
export declare function renderGeometries(indexOffset: number, clusteredCount: number, isStacked: boolean, dataSeries: DataSeries[], xScale: Scale, yScale: Scale, seriesSpecs: Map<SpecId, BasicSeriesSpec>, seriesColorsMap: Map<string, string>, defaultColor: string, axesSpecs: Map<AxisId, AxisSpec>, chartTheme: Theme, enableHistogramMode: boolean): {
    points: PointGeometry[];
    bars: BarGeometry[];
    areas: AreaGeometry[];
    lines: LineGeometry[];
    geometriesIndex: Map<any, IndexedGeometry[]>;
    geometriesCounts: GeometriesCounts;
};
export declare function getSpecById(seriesSpecs: Map<SpecId, BasicSeriesSpec>, specId: SpecId): BasicSeriesSpec | undefined;
export declare function getAxesSpecForSpecId(axesSpecs: Map<AxisId, AxisSpec>, groupId: GroupId): {
    xAxis: AxisSpec | undefined;
    yAxis: AxisSpec | undefined;
};
export declare function computeChartTransform(chartDimensions: Dimensions, chartRotation: Rotation): Transform;
export declare function computeBrushExtent(chartDimensions: Dimensions, chartRotation: Rotation, chartTransform: Transform): BrushExtent;
/**
 * Merge multiple geometry indexes maps together.
 * @param iterables a set of maps to be merged
 * @returns a new Map where each element with the same key are concatenated on a single
 * IndexedGemoetry array for that key
 */
export declare function mergeGeometriesIndexes(...iterables: Map<any, IndexedGeometry[]>[]): Map<any, IndexedGeometry[]>;
export declare function isHorizontalRotation(chartRotation: Rotation): boolean;
export declare function isVerticalRotation(chartRotation: Rotation): boolean;
/**
 * Check if a specs map contains only line or area specs
 * @param specs Map<SpecId, BasicSeriesSpec>
 */
export declare function isLineAreaOnlyChart(specs: Map<SpecId, BasicSeriesSpec>): boolean;
export declare function isChartAnimatable(geometriesCounts: GeometriesCounts, animationEnabled: boolean): boolean;
export declare function isAllSeriesDeselected(legendItems: Map<string, LegendItem>): boolean;
