"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var interactions_1 = require("../utils/interactions");
var rendering_1 = require("../rendering/rendering");
var series_1 = require("../utils/series");
var specs_1 = require("../utils/specs");
var utils_1 = require("../store/utils");
var accessor_1 = require("../../../utils/accessor");
exports.Y0_ACCESSOR_POSTFIX = ' - lower';
exports.Y1_ACCESSOR_POSTFIX = ' - upper';
function getSeriesTooltipValues(tooltipValues, defaultValue) {
    // map from seriesKey to TooltipLegendValue
    var seriesTooltipValues = new Map();
    // First TooltipLegendValue is the header
    if (tooltipValues.length <= 1) {
        return seriesTooltipValues;
    }
    tooltipValues.slice(1).forEach(function (_a) {
        var _b;
        var seriesKey = _a.seriesKey, value = _a.value, yAccessor = _a.yAccessor;
        var seriesValue = defaultValue ? defaultValue : value;
        var current = seriesTooltipValues.get(seriesKey) || {};
        seriesTooltipValues.set(seriesKey, __assign(__assign({ y0: defaultValue, y1: defaultValue }, current), (_b = {}, _b[yAccessor] = seriesValue, _b)));
    });
    return seriesTooltipValues;
}
exports.getSeriesTooltipValues = getSeriesTooltipValues;
function formatTooltip(_a, spec, isXValue, isHighlighted, axisSpec) {
    var color = _a.color, _b = _a.value, x = _b.x, y = _b.y, accessor = _b.accessor, seriesKey = _a.geometryId.seriesKey;
    var seriesKeyAsString = series_1.getColorValuesAsString(seriesKey, spec.id);
    var displayName;
    if (seriesKey.length > 0) {
        displayName = seriesKey.join(' - ');
    }
    else {
        displayName = spec.name || "" + spec.id;
    }
    if (specs_1.isBandedSpec(spec.y0Accessors) && (specs_1.isAreaSeriesSpec(spec) || specs_1.isBarSeriesSpec(spec))) {
        var _c = spec.y0AccessorFormat, y0AccessorFormat = _c === void 0 ? exports.Y0_ACCESSOR_POSTFIX : _c, _d = spec.y1AccessorFormat, y1AccessorFormat = _d === void 0 ? exports.Y1_ACCESSOR_POSTFIX : _d;
        var formatter = accessor === rendering_1.AccessorType.Y0 ? y0AccessorFormat : y1AccessorFormat;
        displayName = accessor_1.getAccessorFormatLabel(formatter, displayName);
    }
    var value = isXValue ? x : y;
    var tickFormatOptions = spec.timeZone ? { timeZone: spec.timeZone } : undefined;
    return {
        seriesKey: seriesKeyAsString,
        name: displayName,
        value: axisSpec ? axisSpec.tickFormat(value, tickFormatOptions) : emptyFormatter(value),
        color: color,
        isHighlighted: isXValue ? false : isHighlighted,
        isXValue: isXValue,
        yAccessor: accessor,
    };
}
exports.formatTooltip = formatTooltip;
function emptyFormatter(value) {
    return value;
}
exports.emptyFormatter = emptyFormatter;
function getTooltipAndHighlightFromXValue(axisCursorPosition, seriesSpecs, axesSpecs, geometriesIndex, xValue, isActiveChart, tooltipType, chartRotation, yScales, tooltipHeaderFormatter) {
    // get the elements on at this cursor position
    var elements = geometriesIndex.get(xValue.value);
    // if no element, hide everything
    if (!elements || elements.length === 0) {
        return;
    }
    // build the tooltip value list
    var xValueInfo = null;
    var highlightedGeometries = [];
    var tooltipData = elements
        .filter(function (_a) {
        var y = _a.value.y;
        return y !== null;
    })
        .reduce(function (acc, indexedGeometry) {
        var specId = indexedGeometry.geometryId.specId;
        var spec = seriesSpecs.get(specId);
        // safe guard check
        if (!spec) {
            return acc;
        }
        var _a = utils_1.getAxesSpecForSpecId(axesSpecs, spec.groupId), xAxis = _a.xAxis, yAxis = _a.yAxis;
        // yScales is ensured by the enclosing if
        var yScale = yScales.get(spec.groupId);
        if (!yScale) {
            return acc;
        }
        // check if the pointer is on the geometry
        var isHighlighted = false;
        if (isActiveChart && rendering_1.isPointOnGeometry(axisCursorPosition.x, axisCursorPosition.y, indexedGeometry)) {
            isHighlighted = true;
            highlightedGeometries.push(indexedGeometry);
        }
        // if it's a follow tooltip, and no element is highlighted
        // not add that element into the tooltip list
        if (!isHighlighted && interactions_1.isFollowTooltipType(tooltipType)) {
            return acc;
        }
        // format the tooltip values
        var yAxisFormatSpec = [0, 180].includes(chartRotation) ? yAxis : xAxis;
        var formattedTooltip = formatTooltip(indexedGeometry, spec, false, isHighlighted, yAxisFormatSpec);
        // format only one time the x value
        if (!xValueInfo) {
            // if we have a tooltipHeaderFormatter, then don't pass in the xAxis as the user will define a formatter
            var xAxisFormatSpec = [0, 180].includes(chartRotation) ? xAxis : yAxis;
            var formatterAxis = tooltipHeaderFormatter ? undefined : xAxisFormatSpec;
            xValueInfo = formatTooltip(indexedGeometry, spec, true, false, formatterAxis);
            return __spread([xValueInfo], acc, [formattedTooltip]);
        }
        return __spread(acc, [formattedTooltip]);
    }, []);
    return {
        tooltipData: tooltipData,
        highlightedGeometries: highlightedGeometries,
    };
}
exports.getTooltipAndHighlightFromXValue = getTooltipAndHighlightFromXValue;
//# sourceMappingURL=tooltip.js.map