import { ColorConfig } from '../../../utils/themes/theme';
import { GroupId, SpecId } from '../../../utils/ids';
import { YBasicSeriesSpec } from '../domains/y_domain';
import { BasicSeriesSpec, Datum, SeriesAccessors } from './specs';
import { LastValues } from '../store/utils';
import { ScaleType } from '../../../utils/scales/scales';
export interface FilledValues {
    /** the x value */
    x: number | string;
    /** the max y value */
    y1: number | null;
    /** the minimum y value */
    y0: number | null;
}
export interface RawDataSeriesDatum {
    /** the x value */
    x: number | string;
    /** the main y metric */
    y1: number | null;
    /** the optional y0 metric, used for bars or area with a lower bound */
    y0?: number | null;
    /** the datum */
    datum?: any;
}
export interface DataSeriesDatum {
    /** the x value */
    x: number | string;
    /** the max y value */
    y1: number | null;
    /** the minimum y value */
    y0: number | null;
    /** initial y1 value, non stacked */
    initialY1: number | null;
    /** initial y0 value, non stacked */
    initialY0: number | null;
    /** the datum */
    datum?: any;
    /** the list of filled values because missing or nulls */
    filled?: Partial<FilledValues>;
}
export interface DataSeries {
    specId: SpecId;
    key: any[];
    seriesColorKey: string;
    data: DataSeriesDatum[];
}
export interface RawDataSeries {
    specId: SpecId;
    key: any[];
    seriesColorKey: string;
    data: RawDataSeriesDatum[];
}
export interface FormattedDataSeries {
    groupId: GroupId;
    dataSeries: DataSeries[];
    counts: DataSeriesCounts;
}
export interface DataSeriesCounts {
    barSeries: number;
    lineSeries: number;
    areaSeries: number;
}
export interface DataSeriesColorsValues {
    specId: SpecId;
    banded?: boolean;
    colorValues: any[];
    lastValue?: LastValues;
    specSortIndex?: number;
}
export declare function findDataSeriesByColorValues(series: DataSeriesColorsValues[] | null, value: DataSeriesColorsValues): number;
/**
 * Split a dataset into multiple series depending on the accessors.
 * Each series is then associated with a key thats belong to its configuration.
 *
 */
export declare function splitSeries(data: Datum[], accessors: SeriesAccessors, specId: SpecId): {
    rawDataSeries: RawDataSeries[];
    colorsValues: Map<string, any[]>;
    xValues: Set<string | number>;
};
/**
 * Get the array of values that forms a series key
 */
export declare function getColorValuesAsString(colorValues: any[], specId: SpecId): string;
export declare function getFormattedDataseries(specs: YBasicSeriesSpec[], dataSeries: Map<SpecId, RawDataSeries[]>, xValues: Set<string | number>, xScaleType: ScaleType): {
    stacked: FormattedDataSeries[];
    nonStacked: FormattedDataSeries[];
};
export declare function getRawDataSeries(seriesSpecs: YBasicSeriesSpec[], dataSeries: Map<SpecId, RawDataSeries[]>): {
    rawDataSeries: RawDataSeries[];
    counts: DataSeriesCounts;
};
/**
 *
 * @param seriesSpecs the map for all the series spec
 * @param deselectedDataSeries the array of deselected/hidden data series
 */
export declare function getSplittedSeries(seriesSpecs: Map<SpecId, BasicSeriesSpec>, deselectedDataSeries?: DataSeriesColorsValues[] | null): {
    splittedSeries: Map<SpecId, RawDataSeries[]>;
    seriesColors: Map<string, DataSeriesColorsValues>;
    xValues: Set<string | number>;
};
export declare function getSortIndex({ specSortIndex }: DataSeriesColorsValues, total: number): number;
export declare function getSortedDataSeriesColorsValuesMap(colorValuesMap: Map<string, DataSeriesColorsValues>): Map<string, DataSeriesColorsValues>;
export declare function getSeriesColorMap(seriesColors: Map<string, DataSeriesColorsValues>, chartColors: ColorConfig, customColors: Map<string, string>): Map<string, string>;
