import { GeometryStyle } from '../../../chart_types/xy_chart/rendering/rendering';
import { Rotation } from '../../../chart_types/xy_chart/utils/specs';
import { AreaStyle, DisplayValueStyle, LineStyle, PointStyle, RectBorderStyle, RectStyle } from '../../../utils/themes/theme';
import { Dimensions } from '../../../utils/dimensions';
import { RectConfig, PathConfig, CircleConfig } from 'konva';
export interface PointStyleProps {
    radius: number;
    stroke: string;
    strokeWidth: number;
    strokeEnabled: boolean;
    fill: string;
    opacity: number;
}
export declare function rotateBarValueProps(chartRotation: Rotation, chartDimensions: Dimensions, barDimensions: Dimensions, displayValueDimensions: Dimensions, displayValue: {
    text: string;
    width: number;
    height: number;
    isValueContainedInElement?: boolean;
}, props: DisplayValueStyle & {
    x: number;
    y: number;
    align: string;
    verticalAlign: string;
    text: string;
    width: number;
    height: number;
}): import("../../../utils/themes/theme").TextStyle & {
    offsetX: number;
    offsetY: number;
} & {
    x: number;
    y: number;
    align: string;
    verticalAlign: string;
    text: string;
    width: number;
    height: number;
};
export declare function getBarValueClipDimensions(displayValue: {
    width: number;
    height: number;
    isValueContainedInElement?: boolean;
}, computedDimensions: {
    width: number;
    height: number;
}, barHeight: number, chartRotation: Rotation): {
    width: number;
    height: number;
    offsetX: number;
    offsetY: number;
};
export declare function isBarValueOverflow(chartDimensions: Dimensions, clip: {
    width: number;
    height: number;
    offsetX: number;
    offsetY: number;
}, valuePosition: {
    x: number;
    y: number;
    offsetX: number;
    offsetY: number;
}, hideClippedValue?: boolean): boolean;
export declare function buildBarValueProps({ x, y, barHeight, barWidth, displayValueStyle, displayValue, chartRotation, chartDimensions, }: {
    x: number;
    y: number;
    barHeight: number;
    barWidth: number;
    displayValueStyle: DisplayValueStyle;
    displayValue: {
        text: string;
        width: number;
        height: number;
        hideClippedValue?: boolean;
        isValueContainedInElement?: boolean;
    };
    chartRotation: Rotation;
    chartDimensions: Dimensions;
}): DisplayValueStyle & {
    x: number;
    y: number;
    align: string;
    text: string;
    width: number;
    height: number;
};
/**
 * Return the style of a point.
 * The color value is used for stroke or fill if they are undefind in the PointStyle
 * @param color the series color
 * @param pointStyle the merged point style
 */
export declare function buildPointStyleProps(color: string, pointStyle: PointStyle, geometryStyle: GeometryStyle): PointStyleProps;
/**
 * Return the rendering props for a point
 * @param x the x position of the point
 * @param y the y position of the point
 * @param pointStyleProps the style props of the point
 */
export declare function buildPointRenderProps(x: number, y: number, pointStyleProps: PointStyleProps): CircleConfig;
/**
 * Return the rendering props for a line. The color of the line will be overwritten
 * by the stroke color of the lineStyle parameter if present
 * @param x the horizontal offset to place the line
 * @param linePath the SVG line path
 * @param color the computed color of the line for this series
 * @param lineStyle the line style
 * @param geometryStyle the highlight geometry style
 */
export declare function buildLineRenderProps(x: number, linePath: string, color: string, lineStyle: LineStyle, geometryStyle: GeometryStyle): PathConfig;
/**
 * Return the rendering props for an area. The color of the area will be overwritten
 * by the fill color of the areaStyle parameter if present
 * @param areaPath the SVG area path
 * @param x the horizontal offset to place the area
 * @param color the computed color of the line for this series
 * @param areaStyle the area style
 * @param geometryStyle the highlight geometry style
 */
export declare function buildAreaRenderProps(xTransform: number, areaPath: string, color: string, areaStyle: AreaStyle, geometryStyle: GeometryStyle): PathConfig;
/**
 * Return the rendering props for a bar. The color of the bar will be overwritten
 * by the fill color of the rectStyle parameter if present
 * @param x the x position of the rect
 * @param y the y position of the rect
 * @param width the width of the rect
 * @param height the height of the rect
 * @param color the computed color of the rect for this series
 * @param rectStyle the rect style
 * @param geometryStyle the highlight geometry style
 */
export declare function buildBarRenderProps(x: number, y: number, width: number, height: number, color: string, rectStyle: RectStyle, borderStyle: RectBorderStyle, geometryStyle: GeometryStyle): RectConfig;
/**
 * Return the rendering props for a bar. The color of the bar will be overwritten
 * by the fill color of the rectStyle parameter if present
 * @param x the x position of the rect
 * @param y the y position of the rect
 * @param width the width of the rect
 * @param height the height of the rect
 * @param color the computed color of the rect for this series
 * @param rectStyle the rect style
 * @param borderStyle the border rect style
 * @param geometryStyle the highlight geometry style
 */
export declare function buildBarBorderRenderProps(x: number, y: number, width: number, height: number, rectStyle: RectStyle, borderStyle: RectBorderStyle, geometryStyle: GeometryStyle): RectConfig | null;
