import { PureComponent } from 'react';
import { DomainRange, Position, Rendering, Rotation } from '../chart_types/xy_chart/utils/specs';
import { PartialTheme, Theme } from '../utils/themes/theme';
import { Domain } from '../utils/domain';
import { TooltipType, TooltipValueFormatter } from '../chart_types/xy_chart/utils/interactions';
import { BrushEndListener, ChartStore, ElementClickListener, ElementOverListener, LegendItemListener, CursorUpdateListener, RenderChangeListener } from '../chart_types/xy_chart/store/chart_state';
import { ScaleTypes } from '../utils/scales/scales';
export declare const DEFAULT_TOOLTIP_TYPE: "vertical";
export declare const DEFAULT_TOOLTIP_SNAP = true;
interface TooltipProps {
    type?: TooltipType;
    snap?: boolean;
    headerFormatter?: TooltipValueFormatter;
}
/**
 * Event used to syncronize cursors between Charts.
 *
 * fired as callback argument for `CursorUpdateListener`
 */
export interface CursorEvent {
    chartId: string;
    scale: ScaleTypes;
    /**
     * @todo
     * unit for event (i.e. `time`, `feet`, `count`, etc.)
     */
    unit?: string;
    value: number | string;
}
export interface SettingSpecProps {
    chartStore?: ChartStore;
    /**
     * Partial theme to be merged with base
     *
     * or
     *
     * Array of partial themes to be merged with base
     * index `0` being the hightest priority
     *
     * i.e. `[primary, secondary, tertiary]`
     */
    theme?: PartialTheme | PartialTheme[];
    /**
     * Full default theme to use as base
     *
     * @default `LIGHT_THEME`
     */
    baseTheme?: Theme;
    rendering: Rendering;
    rotation: Rotation;
    animateData: boolean;
    showLegend: boolean;
    /** Either a TooltipType or an object with configuration of type, snap, and/or headerFormatter */
    tooltip?: TooltipType | TooltipProps;
    debug: boolean;
    legendPosition?: Position;
    showLegendDisplayValue: boolean;
    /**
     * Removes duplicate axes
     *
     * Compares title, position and first & last tick labels
     */
    hideDuplicateAxes: boolean;
    onElementClick?: ElementClickListener;
    onElementOver?: ElementOverListener;
    onElementOut?: () => undefined | void;
    onBrushEnd?: BrushEndListener;
    onLegendItemOver?: LegendItemListener;
    onLegendItemOut?: () => undefined | void;
    onLegendItemClick?: LegendItemListener;
    onLegendItemPlusClick?: LegendItemListener;
    onLegendItemMinusClick?: LegendItemListener;
    onCursorUpdate?: CursorUpdateListener;
    onRenderChange?: RenderChangeListener;
    xDomain?: Domain | DomainRange;
    resizeDebounce?: number;
}
export declare class SettingsComponent extends PureComponent<SettingSpecProps> {
    static defaultProps: Partial<SettingSpecProps>;
    componentDidMount(): void;
    componentDidUpdate(): void;
    render(): null;
}
export declare const Settings: typeof SettingsComponent & import("mobx-react").IWrappedComponent<SettingSpecProps>;
export {};
