const { get, omit } = require('lodash');

const { getConvertedValueForField } = require('./_field_converted_value.js');

function migrateFilter(filter, indexPattern) {

  if (filter && filter.match) {
    const fieldName = Object.keys(filter.match)[0];

    if (isOldStyleMatchPhraseFilter(filter, fieldName)) {
      const params = get(filter, ['match', fieldName]);
      if (get(indexPattern, ['fields'])) {
        const field = indexPattern.fields.find(f => f.name === fieldName);
        if (field) {
          params.query = getConvertedValueForField(field, params.query);
        }
      }
      return {
        match_phrase: {
          [fieldName]: omit(params, 'type'),
        }
      };
    }
  }

  if (get(filter, ['bool', 'should'])) {
    filter.bool.should = Array.isArray(filter.bool.should) ?
      filter.bool.should.map(migrateFilter) : migrateFilter(filter.bool.should);
  }

  if (get(filter, ['bool', 'must'])) {
    filter.bool.must = Array.isArray(filter.bool.must) ?
      filter.bool.must.map(migrateFilter) : migrateFilter(filter.bool.must);
  }

  if (get(filter, ['bool', 'must_not'])) {
    filter.bool.must_not = Array.isArray(filter.bool.must_not) ?
      filter.bool.must_not.map(migrateFilter) : migrateFilter(filter.bool.must_not);
  }

  if (get(filter, ['bool', 'filter'])) {
    filter.bool.filter = Array.isArray(filter.bool.filter) ?
      filter.bool.filter.map(migrateFilter) : migrateFilter(filter.bool.filter);
  }

  return filter;
}

function isOldStyleMatchPhraseFilter(filter, fieldName) {
  return get(filter, ['match', fieldName, 'type']) === 'phrase';
}

module.exports = {
  'migrateFilter': migrateFilter
};