import React, { PureComponent } from 'react';
import { Redirect, Route, Switch } from 'react-router-dom';
import { EuiLink, EuiPage, EuiPageBody, EuiPageContent, EuiPageContentBody, EuiText, EuiTitle } from '@elastic/eui';
import ModelList from './pages/models/ModelList';
import { SavedSearchSelection } from './generic_components';
import HistoricalDetection from './pages/models/HistoricalDetection';
import JobList from './pages/jobs/JobList';
import Header from './pages/Header';
import ModelInfo from './pages/models/ModelInfo';
import ModelTypeSelection from './pages/models/ModelTypeSelection';
import EditModel from './pages/models/EditModel';
const PageNotFound = () => (<div style={{ display: 'flex', flexDirection: 'column', alignItems: 'center' }}>
    <EuiTitle size="m">
      <span>Page not found</span>
    </EuiTitle>
    <EuiText size="m" style={{ marginTop: '10px' }}>
      Maybe you were looking for the <EuiLink href="#/models">model list</EuiLink>?
    </EuiText>
  </div>);
export default class MlRouter extends PureComponent {
    constructor(props) {
        super(props);
        this.updateHeader = ({ currentTab, headerActions }) => this.setState({ currentTab, headerActions });
        this.MakeRoute = (route) => {
            return (<Route path={route.path} render={routeProps => <route.component {...routeProps} {...route.props} updateHeader={this.updateHeader}/>}/>);
        };
        this.state = {
            currentTab: 'models'
        };
        this.routes = [
            {
                path: '/models',
                component: ModelList
            },
            {
                path: '/model/savedSearchSelection',
                component: SavedSearchSelection,
                props: { targetLink: '#/model/new' }
            },
            {
                path: '/model/new',
                component: ModelTypeSelection
            },
            {
                path: ['/model/detection/new', '/model/detection/:id/edit'],
                component: EditModel
            },
            {
                path: ['/model/prediction/new', '/model/prediction/:id/edit'],
                component: EditModel
            },
            {
                path: '/model/:id/view',
                component: ModelInfo
            },
            {
                path: '/model/:id/historicalDetection',
                component: HistoricalDetection
            },
            {
                path: '/jobs',
                component: JobList
            }
        ];
    }
    render() {
        return (<React.Fragment>
        <Header currentTab={this.state.currentTab} headerActions={this.state.headerActions}/>
        <EuiPage>
          <EuiPageBody>
            <EuiPageContent>
              <EuiPageContentBody>
                <Switch>
                  {this.routes.map((route, index) => <this.MakeRoute key={index} {...route}/>)}
                  <Redirect exact path="/" to="/models"/>
                  <Route path="/" component={PageNotFound}/>
                </Switch>
              </EuiPageContentBody>
            </EuiPageContent>
          </EuiPageBody>
        </EuiPage>
      </React.Fragment>);
    }
}
