import moment from 'moment';
import { invert } from 'lodash';
export function getInterval(interval, startTime, endTime) {
    const maxBuckets = 500;
    const delta = endTime.valueOf() - startTime.valueOf();
    const bucketSize = dateMathToMilliseconds(interval);
    if (delta / bucketSize > maxBuckets) {
        const newBucketDuration = delta / maxBuckets;
        let bucketInterval;
        if (newBucketDuration >= moment.duration(1, 'day').valueOf()) {
            bucketInterval = Math.round(moment.duration(newBucketDuration).asDays()) + 'd';
        }
        else if (newBucketDuration >= moment.duration(1, 'hour').valueOf()) {
            bucketInterval = Math.round(moment.duration(newBucketDuration).asHours()) + 'h';
        }
        else if (newBucketDuration >= moment.duration(1, 'minute').valueOf()) {
            bucketInterval = Math.round(moment.duration(newBucketDuration).asMinutes()) + 'm';
        }
        else {
            bucketInterval = Math.round(moment.duration(newBucketDuration).asSeconds()) + 's';
        }
        return bucketInterval;
    }
    else {
        return interval;
    }
}
export function dateMathToMilliseconds(interval) {
    const amount = Number(interval.slice(0, -1));
    const unit = interval.substr(-1);
    return moment.duration(amount, unit).asMilliseconds();
}
export function dateTickFormatter(value, startTime, endTime) {
    const days = (endTime.valueOf() - startTime.valueOf()) / moment.duration(1, 'day').asMilliseconds();
    let format;
    if (days > 30) {
        format = 'YYYY-MM-DD';
    }
    else if (days > 7) {
        format = 'MMMM DD';
    }
    else if (days > 1) {
        format = 'MM-DD HH:mm';
    }
    else {
        format = 'HH:mm:ss';
    }
    return moment(value).format(format);
}
// Date is encoded as the characters that EUI uses for relative times (e.g. date+1d) are automatically encoded
// (date%201d) which screws with the react-router history, making back and forward buttons not work
export class RelativeDateTranscoder {
    static transform(s, mapping) {
        return Object.entries(mapping).reduce((s, [from, to]) => s.replace(from, to), s);
    }
}
RelativeDateTranscoder.mapping = {
    '+': '_plus_',
    '/': '_divide_'
};
RelativeDateTranscoder.encodeDate = (date) => RelativeDateTranscoder.transform(date, RelativeDateTranscoder.mapping);
RelativeDateTranscoder.decodeDate = (date) => RelativeDateTranscoder.transform(date, invert(RelativeDateTranscoder.mapping));
