import React, { Fragment } from 'react';
import { EuiButton, EuiFieldNumber, EuiFlexGroup, EuiFlexItem, EuiFormRow, EuiIconTip, EuiPageContentHeader, EuiPageContentHeaderSection, EuiPanel, EuiTitle } from '@elastic/eui';
import { SelectWithPlaceholder } from '.';
export default class AggregationSelection extends React.PureComponent {
    constructor(props) {
        super(props);
        this.aggregationIsValid = () => !!this.state.aggregationType;
        this.fieldIsValid = () => this.state.aggregationType === 'count' || !!this.state.field;
        this.bucketIsValid = () => !!this.state.bucketSize && this.state.bucketSize > 0 &&
            this.state.bucketSize % 1 === 0 && !!this.state.bucketUnit;
        this.onSubmit = () => {
            this.setState({ showErrors: true });
            if (this.aggregationIsValid() && this.fieldIsValid() && this.bucketIsValid()) {
                this.props.onUpdate({
                    aggregationType: this.state.aggregationType,
                    field: this.state.field,
                    bucketSize: this.state.bucketSize + this.state.bucketUnit
                });
            }
        };
        this.state = {
            bucketSize: props.bucketSize && Number(props.bucketSize.slice(0, -1)),
            bucketUnit: (props.bucketSize && props.bucketSize.substr(-1)),
            aggregationType: props.aggregationType,
            field: props.field,
            showErrors: false
        };
    }
    render() {
        const aggregationTypes = [
            { inputDisplay: 'Count', value: 'count' },
            { inputDisplay: 'Min', value: 'min' },
            { inputDisplay: 'Max', value: 'max' },
            { inputDisplay: 'Sum', value: 'sum' },
            { inputDisplay: 'Average', value: 'avg' }
        ];
        const bucketUnits = [
            { inputDisplay: 'Seconds', value: 's' },
            { inputDisplay: 'Minutes', value: 'm' },
            { inputDisplay: 'Hours', value: 'h' },
            { inputDisplay: 'Days', value: 'd' }
        ];
        return (<EuiPanel>
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle size="xs"><h4>Time Bucketing</h4></EuiTitle>
          </EuiPageContentHeaderSection>
          <EuiPageContentHeaderSection>
            {this.props.actualBucketSize && this.props.actualBucketSize !== this.props.bucketSize && (<EuiIconTip type="alert" color="warning" size="l" position="right" content={`Bucket size is too small to display so ${this.props.actualBucketSize}
                  was used instead. ${this.props.bucketSize} will still be used for training.`}/>)}
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>
        <EuiFormRow data-test="aggregation-drop-down" label="Aggregation" isInvalid={this.state.showErrors && !this.aggregationIsValid()} error="An aggregation type must be selected">
          <SelectWithPlaceholder placeholder="Select an aggregation type" value={this.state.aggregationType} options={aggregationTypes} onChange={aggregationType => this.setState({ aggregationType })}/>
        </EuiFormRow>
        <EuiFormRow label={<Fragment>
              Field <EuiIconTip content="The field used as input to the model and where anomalies are detected" position="right"/>
            </Fragment>} data-test="predict-field-drop-down" isInvalid={this.state.showErrors && !this.fieldIsValid()} error="A field must be selected if not using the 'count' aggregation">
          <SelectWithPlaceholder placeholder="Select a field" value={this.state.aggregationType !== 'count' ? this.state.field : undefined} options={this.props.fields.map(field => ({ inputDisplay: field, value: field }))} isDisabled={this.state.aggregationType === 'count'} onChange={field => this.setState({ field })}/>
        </EuiFormRow>
        <EuiFormRow label="Bucket size" isInvalid={this.state.showErrors && !this.bucketIsValid()} error="A valid bucket size must be specified">
          <EuiFlexGroup gutterSize="none">
            <EuiFlexItem style={{ marginRight: '12px' }}>
              <EuiFieldNumber data-test="bucket-size-field" value={this.state.bucketSize} placeholder="Size" onChange={e => this.setState({ bucketSize: isNaN(parseFloat(e.target.value)) ? '' : parseFloat(e.target.value) })} min={1}/>
            </EuiFlexItem>
            <EuiFlexItem data-test="bucket-unit-drop-down" style={{ marginLeft: '12px' }}>
              <SelectWithPlaceholder placeholder="Unit" value={this.state.bucketUnit} options={bucketUnits} onChange={bucketUnit => this.setState({ bucketUnit })}/>
            </EuiFlexItem>
          </EuiFlexGroup>
        </EuiFormRow>
        <EuiFormRow style={{ marginTop: 20 }}>
          <EuiButton iconType="play" fill data-test="play-btn" onClick={this.onSubmit}>
            Apply
          </EuiButton>
        </EuiFormRow>
      </EuiPanel>);
    }
}
