import React from 'react';
import { EuiPanel, EuiFormRow, EuiFieldNumber, EuiTitle, EuiPageContentHeader, EuiPageContentHeaderSection } from '@elastic/eui';
export default class PredictionSettings extends React.PureComponent {
    render() {
        return (<EuiPanel>
        <EuiPageContentHeader style={{ marginBottom: 12 }}>
          <EuiPageContentHeaderSection>
            <EuiTitle size="xs"><h4>Model Settings</h4></EuiTitle>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>

        {this.getSettings()}
      </EuiPanel>);
    }
    getSettings() {
        if (this.props.interval) {
            return (<OutputSize value={this.props.bucketCount} interval={this.props.interval} onChange={this.props.onSizeChange}/>);
        }
        else {
            return 'Fill in the parameters above to see settings.';
        }
    }
}
const isValidOutputSize = (value) => !!value && value % 1 === 0 && value > 0;
const OutputSize = (props) => {
    const bucketSize = Number(props.interval.slice(0, -1));
    const unit = {
        s: 'second',
        m: 'minute',
        h: 'hour',
        d: 'day'
    }[props.interval.substr(-1)];
    let text = '';
    if (isValidOutputSize(props.value)) {
        text = `Model will predict ${props.value * bucketSize} ${unit}${props.value * bucketSize > 1 ? 's' : ''} into the future.`;
    }
    return (<EuiFormRow label="Future buckets to predict" helpText={text} isInvalid={!isValidOutputSize(props.value)} error={props.value && props.value % 1 !== 0 ? 'Must be a whole number' : 'Must be a number greater than zero'}>
      <EuiFieldNumber data-test="prediction-size-input" value={props.value || ''} placeholder="Bucket count" onChange={e => props.onChange && props.onChange(e.target.value === '' ? undefined : Number(e.target.value))} min={1}/>
    </EuiFormRow>);
};
