import React, { PureComponent } from 'react';
import { EuiIcon, EuiInMemoryTable, EuiLink, EuiPageContentHeader, EuiPageContentHeaderSection, EuiTitle, EuiToolTip } from '@elastic/eui';
// @ts-ignore
import { setMlPageTitle } from '../constants';
import { AppDependencies } from '../AngularJsDependencies';
export default class SavedSearchSelection extends PureComponent {
    constructor(props) {
        super(props);
        this.state = {};
        props.updateHeader({});
        this.columns = [{
                field: 'hasTimeField',
                name: 'Saved Search',
                sortable: true,
                render: (hasTimeField, search) => {
                    if (hasTimeField) {
                        return (<EuiLink href={`${props.targetLink}?savedSearch=${search.id}`} data-test={search.title}>
              {search.title}
            </EuiLink>);
                    }
                    else {
                        return (<EuiToolTip position="right" content="No time field associated">
              <span style={{ cursor: 'not-allowed' }}>{search.title} <EuiIcon type="clock" color="danger"/></span>
            </EuiToolTip>);
                    }
                }
            }];
        setMlPageTitle('Saved Search Selection');
    }
    componentDidMount() {
        this.fetchIndexPatterns();
    }
    fetchIndexPatterns() {
        this.context.savedObjectsClient.find({ type: 'search' })
            .then(response => response.savedObjects)
            .then(async (savedSearches) => {
            return Promise.all(savedSearches.map(async (savedSearch) => {
                const indexId = JSON.parse(savedSearch.attributes.kibanaSavedObjectMeta.searchSourceJSON).index;
                const indexPattern = await this.context.savedObjectsClient.get('index-pattern', indexId);
                return {
                    id: savedSearch.id,
                    title: savedSearch.attributes.title,
                    hasTimeField: !!indexPattern.attributes.timeFieldName
                };
            }));
        })
            .then(savedSearches => this.setState({ savedSearches }))
            .catch(error => this.setState({ error: error.message }));
    }
    render() {
        return (<div className="wizard-column wizard-column--large">
        <EuiPageContentHeader>
          <EuiPageContentHeaderSection>
            <EuiTitle>
              <h2 data-test="search-title">Pick a Saved Search</h2>
            </EuiTitle>
          </EuiPageContentHeaderSection>
        </EuiPageContentHeader>

        <EuiInMemoryTable items={this.state.savedSearches ? this.state.savedSearches : []} columns={this.columns} loading={!this.state.savedSearches && !this.state.error} noItemsMessage={!this.state.savedSearches ? 'Retrieving saved searches...' : 'No saved searches found'} error={this.state.error} 
        // @ts-ignore
        search={{ box: { incremental: true, 'data-test': 'input-saved-searches' } }} pagination={true} sorting={true} data-test="saved-searches-table"/>
      </div>);
    }
}
SavedSearchSelection.contextType = AppDependencies;
