import React, { PureComponent } from 'react';
// @ts-ignore
// eslint-disable-next-line import/no-unresolved
import { uiModules } from 'ui/modules';
import '@elastic/eui/dist/eui_theme_light.css';
import { render, unmountComponentAtNode } from 'react-dom';
import { EuiFlexGroup, EuiFlexItem, EuiFormRow } from '@elastic/eui';
import { SelectWithPlaceholder } from '../generic_components';
import { uniq } from 'lodash';
import './vis_styles.less';
import { ModelService } from '../services';
// @ts-ignore
import { parseWithPrecision } from 'ui/kibi/utils/date_math_precision'; // eslint-disable-line import/no-unresolved
// @ts-ignore
import { SavedObjectsClientProvider } from 'ui/saved_objects'; // eslint-disable-line import/no-unresolved
import moment from 'moment';
import { ExplorerVisGraph } from './ExplorerVisGraph';
import { ExplorerDependencies } from '../AngularJsDependencies';
uiModules
    .get('kibana/explorer_vis', ['kibana'])
    .directive('explorerVis', () => ({ restrict: 'E', controller: ExplorerVisWrapper }));
function ExplorerVisWrapper($scope, $element, $rootScope, createNotifier, timefilter, Private) {
    render(<ExplorerDependencies.Provider value={{
        rootScope: $rootScope,
        notifier: createNotifier('Explorer vis'),
        timefilter,
        savedObjectsClient: Private(SavedObjectsClientProvider)
    }}>
      <ExplorerVis />
    </ExplorerDependencies.Provider>, $element[0]);
    $scope.$on('$destroy', () => unmountComponentAtNode($element[0]));
}
export class ExplorerVis extends PureComponent {
    constructor(props) {
        super(props);
        this.updateTimeRange = () => {
            if (this.state.selectedModel) {
                this.setState({
                    startTime: parseWithPrecision(this.context.timefilter.time.from, false, this.context.rootScope.sirenTimePrecision),
                    endTime: parseWithPrecision(this.context.timefilter.time.to, false, this.context.rootScope.sirenTimePrecision)
                });
            }
        };
        this.onSavedSearchSelect = (selectedSavedSearch) => {
            this.setState({ selectedSavedSearch, selectedModel: undefined });
        };
        this.onModelSelect = (selectedModel) => this.setState({ selectedModel: this.state.models.find(model => model.name === selectedModel) });
        this.handleSelectionBrushEnd = (startTime, endTime) => {
            this.context.timefilter.time = {
                from: moment(startTime).toISOString(),
                to: moment(endTime).toISOString(),
                mode: 'absolute'
            };
        };
        this.state = {
            models: [],
            isLoading: true,
            savedSearches: []
        };
    }
    componentDidMount() {
        this.setState({
            startTime: parseWithPrecision(this.context.timefilter.time.from, false, this.context.rootScope.sirenTimePrecision),
            endTime: parseWithPrecision(this.context.timefilter.time.to, false, this.context.rootScope.sirenTimePrecision)
        });
        this.fetchModels();
        this.context.timefilter.on('update', this.updateTimeRange);
        this.searchRefreshEventDeregister = this.context.rootScope.$on('courier:searchRefresh', this.updateTimeRange);
    }
    componentWillUnmount() {
        this.context.timefilter.off('update', this.updateTimeRange);
        this.searchRefreshEventDeregister();
    }
    fetchModels() {
        ModelService.getAll()
            .then(models => Promise.all(models.map(m => ModelService.get(m.name))))
            .then(async (models) => {
            const savedSearchNames = uniq(models.map(model => model.data.savedSearch));
            const savedSearches = await Promise.all(savedSearchNames.map(savedSearch => this.context.savedObjectsClient.get('search', savedSearch)));
            this.setState({
                models,
                savedSearches: savedSearches.map(savedSearch => ({
                    value: savedSearch.id,
                    inputDisplay: savedSearch.attributes.title
                }))
            });
        })
            .catch(err => this.context.notifier.error('Could not retrieve models: ' + err.message));
    }
    render() {
        const models = this.state.models
            .filter(model => model.data.savedSearch === this.state.selectedSavedSearch)
            .map(model => ({
            value: model.name,
            inputDisplay: model.name + (model.training.state !== 'trained' ? ` (${model.training.state})` : ''),
            disabled: model.training.state !== 'trained'
        }));
        return (<div style={{ padding: '20px 10px', height: 'calc(100% - 40px)' }}>
        <EuiFlexGroup wrap={true} alignItems="flexEnd" style={{ paddingBottom: '10px', width: '100%' }}>
          <EuiFlexItem style={{ minWidth: '150px' }}>
            <EuiFormRow label="Saved search" data-test="ml-saved-search-drop-down">
              <SelectWithPlaceholder value={this.state.selectedSavedSearch} options={this.state.savedSearches} placeholder="Select a saved search" onChange={this.onSavedSearchSelect}/>
            </EuiFormRow>
          </EuiFlexItem>
          <EuiFlexItem style={{ minWidth: '150px' }}>
            <EuiFormRow label="Model" data-test="ml-model-drop-down">
              <SelectWithPlaceholder value={this.state.selectedModel && this.state.selectedModel.name} options={models} placeholder="Select a model" onChange={this.onModelSelect}/>
            </EuiFormRow>
          </EuiFlexItem>
        </EuiFlexGroup>
        {this.state.selectedModel && this.state.startTime && this.state.endTime &&
            <ExplorerVisGraph model={this.state.selectedModel} startTime={this.state.startTime} endTime={this.state.endTime} onBrushEnd={this.handleSelectionBrushEnd}/>}
      </div>);
    }
}
ExplorerVis.contextType = ExplorerDependencies;
