"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graph_control_1 = require("./saved_objects/graph_control");
const dashboard_1 = require("./saved_objects/dashboard");
class ExplorerSetup {
    static async setup(server) {
        const savedObjectsClient = server.savedObjectsClientFactory({
            callCluster: server.plugins.elasticsearch.getCluster('admin')
        });
        const creds = server.plugins.saved_objects_api.getServerCredentials();
        // Changes in any of the IDs must be reflected in dependent saved object definition files (e.g. the graph control
        // is specified in the dashboard).
        await ExplorerSetup.createSavedObject('visualization:ml-graph-controls', graph_control_1.default, savedObjectsClient, creds);
        await ExplorerSetup.createSavedObject('dashboard:ml-explorer', dashboard_1.default, savedObjectsClient, creds);
        try {
            await ExplorerSetup.updateDashboardGroup('dashboard:ml-explorer', savedObjectsClient, creds);
        }
        catch (err) {
            if (err.statusCode === 404) {
                server.log(['info', 'machine_learning'], 'Dashboard group with Graph Browser not found, so can not add');
            }
            else {
                throw err;
            }
        }
    }
    static async createSavedObject(id, definition, savedObjectsClient, serverCredentials) {
        const type = id.substr(0, id.indexOf(':'));
        try {
            await savedObjectsClient.get(type, id, serverCredentials);
        }
        catch (err) {
            if (err.statusCode === 404) {
                try {
                    await savedObjectsClient.create(type, definition, { id }, serverCredentials);
                }
                catch (err) {
                    err.message = `Could not create saved object ${id}: ${err.message}`;
                    throw err;
                }
            }
            else {
                err.message = `Could not retrieve saved object ${id}: ${err.message}`;
                throw err;
            }
        }
    }
    static async updateDashboardGroup(dashboardId, savedObjectsClient, serverCredentials) {
        const toolsDashboardGroupId = 'dashboardgroup:Graph-Browser';
        const dashboardGroup = await savedObjectsClient.get('dashboardgroup', toolsDashboardGroupId, serverCredentials);
        const dashboards = JSON.parse(dashboardGroup.attributes.dashboards);
        if (!dashboards.find(dashboard => dashboard.id === dashboardId)) {
            dashboards.push({
                id: dashboardId,
                title: 'ML Explorer'
            });
            dashboardGroup.attributes.dashboards = JSON.stringify(dashboards);
            await savedObjectsClient.update('dashboardgroup', toolsDashboardGroupId, dashboardGroup.attributes, null, serverCredentials);
        }
    }
}
exports.default = ExplorerSetup.setup;
