"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const explorer_setup_1 = require("./explorer_setup");
const ModelRoutes_1 = require("../routes/ModelRoutes");
const JobRoutes_1 = require("../routes/JobRoutes");
const SirenMlErrorHandler_1 = require("../routes/SirenMlErrorHandler");
const Routes_1 = require("../routes/Routes");
class ServerInitialisation {
    static async initialiseServer(server) {
        try {
            await explorer_setup_1.default(server);
            await ServerInitialisation.setupRoutes(server);
            this.status.green('Ready');
        }
        catch (error) {
            this.status.red('Failed to start up: ' + error.message);
            console.error(error);
        }
    }
    static async setupRoutes(server) {
        await server.register(Routes_1.hapiPluginFromClass(ModelRoutes_1.default), { routes: { prefix: '/api/machine_learning/models' } });
        await server.register(Routes_1.hapiPluginFromClass(JobRoutes_1.default), { routes: { prefix: '/api/machine_learning/jobs' } });
        server.register(SirenMlErrorHandler_1.default);
    }
}
exports.default = ServerInitialisation.initialiseServer;
