"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const GlobalConfiguration_1 = require("./GlobalConfiguration");
const Models_1 = require("./Models");
const Deployment_1 = require("./Deployment");
const Jobs_1 = require("./Jobs");
const Errors_1 = require("./Errors");
const debug_1 = require("debug");
const debug = debug_1.default('machine-learning:sirenml');
class SirenMl {
    constructor(uri, credentials) {
        this.uri = uri;
        this.http = SirenMl.getHttpService(this.uri, credentials);
        this.globalConfig = new GlobalConfiguration_1.default(this, this.http);
        this.models = new Models_1.default(this, this.http);
        this.deployment = new Deployment_1.default(this, this.http);
        this.jobs = new Jobs_1.default(this, this.http);
    }
    static getHttpService(uri, credentials) {
        const http = axios_1.default.create({
            baseURL: uri,
            timeout: 10000,
            headers: { 'Content-Type': 'application/json' },
            auth: credentials
        });
        http.interceptors.request.use(request => {
            debug(request.method && request.method.toUpperCase(), request.url);
            return request;
        });
        http.interceptors.response.use(response => response, err => {
            var _a, _b, _c;
            const code = err.response ? err.response.status : err.code;
            debug('Status: ', code);
            debug('Error: %p', err.response && err.response.data || err.message);
            const errorInfo = ((_b = (_a = err.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) || ((_c = err.response) === null || _c === void 0 ? void 0 : _c.data) || err.message;
            const message = 'Error returned from Siren ML: ' + typeof errorInfo === 'object' ? JSON.stringify(errorInfo) : errorInfo;
            throw new Errors_1.SirenMlError(code, message);
        });
        return http;
    }
}
exports.default = SirenMl;
debug_1.default.formatters.p = object => {
    if (typeof object !== 'string') {
        object = JSON.stringify(object, undefined, 2);
    }
    return object.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
        let cls = '\x1b[36m';
        if (/^"/.test(match)) {
            if (/:$/.test(match)) {
                cls = '\x1b[33m';
            }
            else {
                cls = '\x1b[32m';
            }
        }
        else if (/true|false/.test(match)) {
            cls = '\x1b[35m';
        }
        else if (/null/.test(match)) {
            cls = '\x1b[31m';
        }
        return cls + match + '\x1b[0m';
    });
};
